#include "type.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define SIZE 5
#define ORIGIN_FILE "origin.txt"
#define STACK_FILE  "stack.txt"

struct elem v[SIZE];

void initialize();
int menu();

void load();
void display(FILE *out);

// functions to handle each menu option
void sort_option();
void search_option();
void stack_option();
void linkedlist_option_1();
void linkedlist_option_2();


int main() {

  initialize();

  while (1) {
    int option = menu();
    switch (option) {
      case 1: // sort
        sort_option();
        break;

      case 2: // search
        search_option();
        break;

      case 3: // insert to stack
        stack_option();
        break;

      case 4: // insert to linked list
        linkedlist_option_1();
        break;

      case 5: // insert one more element to linked list
        linkedlist_option_2();
        break;

      case 6: // quit
        return 1;
    }
    printf("\n\n");
  }

  return 0;
}

// load the elements from the text file into array
void load() {
  FILE *f;
  int i;

  f = fopen(ORIGIN_FILE, "r");
  if (f == NULL) {
    printf("could not read file %s.\n", ORIGIN_FILE);
    exit(-1);
  }

  /* load each array element (structure) */
  for (i = 0; i < SIZE; i++) {
    fscanf(f, "%s %d %f", v[i].arr, &v[i].v1, &v[i].v2);
  }

  fclose(f);
}

void display(FILE *out) {
  int i;
  for (i = 0; i < SIZE; i++) {
    display_elem(out, &v[i]);
  }
}

// functions to handle each menu option
void sort_option() {
  int option;

  // prompt the user how to sort the array
  printf("\nSort options:\n");
  printf("1. ascending / int value / insertion sort\n");
  printf("2. ascending / int value / selection sort\n");
  printf("3. ascending / float value / insertion sort\n");
  printf("4. ascending / float value / selection sort\n");
  printf("5. descending / int value / insertion sort\n");
  printf("6. descending / int value / selection sort\n");
  printf("7. descending / float value / insertion sort\n");
  printf("8. descending / float value / selection sort\n");
  printf("Choose: ");
  scanf("%d", &option);
  while (option < 1 || option > 8) {
    printf("Invalid. Choose again: ");
    scanf("%d", &option);
  }
  printf("\n\n");

  /* load from the text file */
  load();

  /* call the corresponding sorting algorithm */
  switch(option) {
    case 1:
      insertion_sort(v, SIZE, 0, 0);
      break;
    case 2:
      selection_sort(v, SIZE, 0, 0);
      break;
    case 3:
      insertion_sort(v, SIZE, 1, 0);
      break;
    case 4:
      selection_sort(v, SIZE, 1, 0);
      break;
    case 5:
      insertion_sort(v, SIZE, 0, 1);
      break;
    case 6:
      selection_sort(v, SIZE, 0, 1);
      break;
    case 7:
      insertion_sort(v, SIZE, 1, 1);
      break;
    case 8:
      selection_sort(v, SIZE, 1, 1);
      break;
  }

  /* display the sort result */
  display(stdout);
  printf("\n\n");
}

void search_option() {
  int option;
  int value;

  // prompt the user how to search the array
  printf("\nSearch options:\n");
  printf("1. linear search\n");
  printf("2. binary search\n");
  printf("Choose: ");
  scanf("%d", &option);
  while (option < 1 || option > 2) {
    printf("Invalid. Choose again: ");
    scanf("%d", &option);
  }
  printf("\n\n");

  printf("Enter the value to search: ");
  scanf("%d", &value);

  /* load from the text file */
  load();

  /* call the correspoding search functions */
  if (option == 1) {
    linear_search(v, SIZE, value);
  } else {
    binary_search(v, SIZE, value);
  }
}

void stack_option() {
  int i;
  int option;
  struct stack s;

  /* load from the text file */
  load();

  /* insert array into a stack */
  s.top = NULL;
  for (i = 0; i < SIZE; i++) {
    push_stack(&s, v[i]);
  }

  /* search by char array or int */
  printf("\nSearch options:\n");
  printf("1. search a char array\n");
  printf("2. search an int element\n");
  printf("Choose: ");
  scanf("%d", &option);
  while (option < 1 || option > 2) {
    printf("Invalid. Choose again: ");
    scanf("%d", &option);
  }
  printf("\n\n");

  /* linear search in the stack */
  if (option == 1) {
    char value[128];
    printf("Enter the char array to search: ");
    scanf("%s", value);
    linear_search_stack_1(&s, value);
  } else {
    int value;
    printf("Enter the int value to search: ");
    scanf("%d", &value);
    linear_search_stack_2(&s, value);
  }

  /* ask the user if save the stack into a file */
  printf("\nEnter 1 to save into %s in reverse order, 0 to stop: ", STACK_FILE);
  scanf("%d", &option);
  if (option == 1) {
    FILE *f;
    f = fopen(STACK_FILE, "w");
    if (f == NULL) {
      printf("could not write into %s.\n", STACK_FILE);
      exit(-1);
    }
    save_stack(f, &s);
    fclose(f);
    printf("The stack is saved into file %s.\n", STACK_FILE);
  }
  printf("\n");
}

void linkedlist_option_1() {
  int i;
  struct linkedlist l;

  /* load from the text file */
  load();

  /* insert into the list */
  l.head = NULL;
  for (i = 0; i < SIZE; i++) {
    insert_linkedlist(&l, v[i]);
  }

  display_linkedlist(stdout, &l);
  printf("\n\n");
}

void linkedlist_option_2() {
  int i;
  struct linkedlist l;
  struct elem e;

  /* load from the text file */
  load();

  /* insert into the list */
  l.head = NULL;
  for (i = 0; i < SIZE; i++) {
    insert_linkedlist(&l, v[i]);
  }

  /* ask for a new element and insert into the list in proper position */
  printf("\nEnter the values for new structure\n");
  printf("char array (no space): ");
  scanf("%s", e.arr);
  printf("int value: ");
  scanf("%d", &e.v1);
  printf("float value: ");
  scanf("%f", &e.v2);
  insert_linkedlist(&l, e);

  display_linkedlist(stdout, &l);
  printf("\n\n");
}

int menu() {
  int choice;
  // display the menu and return the user's option
  printf("\n\n1. sort the array\n");
  printf("2. search for an int element\n");
  printf("3. insert into a stack\n");
  printf("4. insert the sorted elements into a linked list\n");
  printf("5. insert one more structure into the linked list\n");
  printf("6. Quit\n");
  printf("Choose: ");

  // ask the user to choose one
  scanf("%d", &choice);
  while (choice < 1 || choice > 6) {
    printf("Invalid. Choose again: ");
    scanf("%d", &choice);
  }
  printf("\n\n");
  return choice;
}

void initialize() {
  FILE *f;
  int i;

  // prompt the user to enter the information of the array of structure
  // and save into a text file
  for (i = 0; i < SIZE; i++) {
    printf("\nEnter the values for structure %d\n", i + 1);
    printf("char array (no space): ");
    scanf("%s", v[i].arr);
    printf("int value: ");
    scanf("%d", &v[i].v1);
    printf("float value: ");
    scanf("%f", &v[i].v2);
  }

  /* save into the structure into a file */
  f = fopen(ORIGIN_FILE, "w");
  if (f == NULL) {
    printf("could not save into %s.\n", ORIGIN_FILE);
    exit(-1);
  }
  display(f);
  fclose(f);

  printf("\n\nThe structure is saved into %s.\n\n", ORIGIN_FILE);
}
