#include "type.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* implementation of the functions in type.h */

void display_elem(FILE *out, struct elem *e) {
  fprintf(out, "%-32s %-10d %f\n", e->arr, e->v1, e->v2);
}

/* two sorting functions: bit to specify to sort using
 * int value or float value. flag to specify ascending or descending */
void insertion_sort(struct elem v[], int n, int bit, int flag) {
  int i;
  for (i = 1; i < n; i++) {
    struct elem e = v[i]; /* insert element e into the proper position */
    int k = i - 1;
    if (bit == 0) { /* sort by int value */
      while (k >= 0 && v[k].v1 > e.v1) {
        v[k + 1] = v[k];
        k --;
      }
    } else { /* sort  by float value */
      while (k >= 0 && v[k].v2 > e.v2) {
        v[k + 1] = v[k];
        k --;
      }
    }
    v[k + 1] = e;
  }

  if (flag == 1) { /* reverse to descending */
    reverse(v, n);
  }
}

void selection_sort(struct elem v[], int n, int bit, int flag) {
  int i;
  for (i = 0; i < n - 1; i++) {
    /* select the one in the range [i, n) to put at the location i. */
    int k = i;
    int j;
    for (j = i + 1; j < n; j++) {
      if (bit == 0) { /* sort by int value */
        if (v[j].v1 < v[k].v1) {
          k = j;
        }
      } else { /* sort by float value */
        if (v[j].v2 < v[k].v2) {
          k = j;
        }
      }
    }
    if (k != i) {
      /* put the selected value to the location i */
      struct elem e = v[k];
      v[k] = v[i];
      v[i] = e;
    }
  }
  if (flag == 1) { /* reverse to descending */
    reverse(v, n);
  }
}

void reverse(struct elem v[], int n) {
  int i = 0, j = n - 1;
  while (i < j) {
    /* swap the position of two elements */
    struct elem e = v[i];
    v[i] = v[j];
    v[j] = e;
    i ++;
    j --;
  }
}

/* two search functions */
void linear_search(struct elem v[], int n, int value) {
  int i;
  for (i = 0; i < n; i++) {
    if (v[i].v1 == value) {
      printf("FOUND:\n");
      display_elem(stdout, &v[i]);
      return;
    }
  }
  printf("NOT FOUND\n");
}

void binary_search(struct elem v[], int n, int value) {
  int l = 0, r = n;

  /* sort the element ascending by int value before binary search */
  insertion_sort(v, n, 0, 0);

  while (l < r) {
    int m = l + (r - l) / 2;
    if (v[m].v1 < value) { /* look into the right part */
      l = m + 1;
    } else if (v[m].v1 > value) { /* look into the left part */
      r = m;
    } else { /* found */
      printf("FOUND:\n");
      display_elem(stdout, &v[m]);
      return;
    }
  }
  printf("NOT FOUND\n");
}

/* list functions */
void insert_linkedlist(struct linkedlist *l, struct elem e) {
  struct node *n;

  /* find the location in the linked list to insert the element ascendingly by
   * the int value */
  struct node **p = &l->head;
  while (*p != NULL && (*p)->e.v1 <= e.v1) {
    p = &(*p)->next;
  }

  /* insert into this location */
  n = (struct node *)malloc(sizeof(struct node));
  n->e = e;
  n->next = *p;
  *p = n;
}

void display_linkedlist(FILE *out, struct linkedlist *l) {
  struct node *n = l->head;
  while (n != NULL) { /* check each node in the list s*/
    display_elem(out, &n->e);
    n = n->next;
  }
}

/* stack functions */
void push_stack(struct stack *s, struct elem e) {
  struct node *n;

  /* add to the top of the stack */
  n = (struct node *)malloc(sizeof(struct node));
  n->e = e;
  n->next = s->top;
  s->top = n;
}

void linear_search_stack_1(struct stack *s, char *value) {
  struct node *n = s->top;

  while (n != NULL) { /* check each element on the stack */
    if (strcmp(n->e.arr, value) == 0) {
      printf("FOUND:\n");
      display_elem(stdout, &n->e);
      return;
    }
    n = n->next;
  }
  printf("NOT FOUND\n");
}

void linear_search_stack_2(struct stack *s, int value) {
  struct node *n = s->top;

  while (n != NULL) { /* check each element on the stack */
    if (n->e.v1 == value) {
      printf("FOUND:\n");
      display_elem(stdout, &n->e);
      return;
    }
    n = n->next;
  }
  printf("NOT FOUND\n");
}


void save_stack(FILE *out, struct stack *s) {
  struct node *n = s->top;
  while (n != NULL) { /* check each node in the stack */
    display_elem(out, &n->e);
    n = n->next;
  }
}

